-------------------------------------------------------------------------------
-- ElvUI Currency Datatext By Lockslap
-------------------------------------------------------------------------------
local E, _, V, P, G = unpack(ElvUI)
local DT = E:GetModule("DataTexts")
local L = LibStub("AceLocale-3.0"):GetLocale("ElvUI_CurrencyDatatext", false)
local EP = LibStub("LibElvUIPlugin-1.0")

local wipe = table.wipe
local sort = table.sort
local join = string.join

--local displayString = ""
local hexColor = ""
local bgs = {
	[1] = L["2v2"],
	[2] = L["3v3"],
	[3] = L["5v5"],
	[4] = L["10v10 (RBGs)"],
}

local function SortCurrencies()
	local temp, curHeader = {}, ""
	if GetCurrencyListSize() == 0 then return false end
	for i = 1, GetCurrencyListSize() do
		local name, isHeader, _, isUnused, isWatched, count, icon = GetCurrencyListInfo(i)
		if isHeader then
			temp[name] = {}
			curHeader = name
		else
			temp[curHeader][#temp[curHeader] + 1] = {
				["name"] = name,
				["count"] = count,
				["icon"] = icon,
				["watched"] = isWatched,
				["unused"] = isUnused,
			}
		end
	end
	return temp
end

local function OnEvent(self, event)
	-- make sure the pvp frame is loaded for ratings
	local displayString = ""
	if event == "PLAYER_ENTERING_WORLD" and PVPUIFrame == nil then
		LoadAddOn("Blizzard_PVPUI")
	end
	
	local name, _, _, _, _, count, icon = GetCurrencyListInfo(E.db.currencydt.display)
	
	-- build the icon/text string based on config
	if E.db.currencydt.icon == true then
		displayString = displayString .. ("|T%s:14:14:0:0:64:64:4:60:4:60|t"):format(icon)
	end
	if E.db.currencydt.showText == true then
		displayString = displayString .. ("%s|cffffffff%s%s|r"):format(E.db.currencydt.icon == true and " " or "", name, E.db.currencydt.showValue == true and ":" or "")
	end
	if E.db.currencydt.showValue == true then
		displayString = displayString .. ("%s%s%d|r"):format(E.db.currencydt.icon == true and " " or E.db.currencydt.showText == true and " " or "", hexColor, count)
	end
	if not displayString or displayString == "" then
		displayString = ("%s%s|r"):format(hexColor, L["Nothing to Display"])
	end
	--self.text:SetFormattedText(displayString, E.db.currencydt.icon == true and  or "", E.db.currencydt.showText == true and name or "", count)
	self.text:SetText(displayString)
end

local function OnEnter(self)
	local currencies = SortCurrencies()
	local currencyString = "|T%s:14:14:0:0:64:64:4:60:4:60|t |cff%s%s|r"
	local color = "ffffff"
	local countColor = "ffff00"
	DT:SetupTooltip(self)
	
	if not currencies then
		DT.tooltip:AddLine(L["No Currencies Found"])
	else
		for header, items in pairs(currencies) do
			if E.db.currencydt.headers[header] then
				
				if E.db.currencydt.tooltip.headers then
					DT.tooltip:AddLine(header)
				end
				
				for _, currency in ipairs(items) do
					if E.db.currencydt.tooltip.zeroCurrency ~= "hide" or currency.count ~= 0 then
						if currency.watched then
							color = E.db.currencydt.tooltip.colorWatched and ("%02x%02x%02x"):format(E.db.currencydt.tooltip.watchedColor.r * 255, E.db.currencydt.tooltip.watchedColor.g * 255, E.db.currencydt.tooltip.watchedColor.b * 255) or "ffffff"
						else
							if E.db.currencydt.tooltip.zeroCurrency == "gray" then
								color = currency.count == 0 and "505050" or "ffffff"
								countColor = currency.count == 0 and "505050" or "ffff00"
							else
								color = "ffffff"
								countColor = "ffff00"
							end
						end
						DT.tooltip:AddDoubleLine(currencyString:format(currency.icon, color, currency.name), ("|cff%s%d|r"):format(countColor, currency.count))
					end
				end
				
				if E.db.currencydt.tooltip.headers then
					DT.tooltip:AddLine(" ")
				end
			end
		end
	end
	
	-- arena/rbg ratings
	if E.db.currencydt.tooltip.ratings == true then
		if IsShiftKeyDown() then
			DT.tooltip:AddLine(L["Best Weekly Arena/RBG Ratings"])
		elseif IsControlKeyDown() then
			DT.tooltip:AddLine(L["Best Season Arena/RBG Ratings"])
		else
			DT.tooltip:AddLine(L["Current Arena/RBG Ratings"])
		end
		
		for i = 1, 4 do
			local rating, seasonBest, weeklyBest = GetPersonalRatedInfo(i)
			if IsShiftKeyDown() then
				DT.tooltip:AddDoubleLine(bgs[i], weeklyBest, 1, 1, 1, 1, 1, 0)
			elseif IsControlKeyDown() then
				DT.tooltip:AddDoubleLine(bgs[i], seasonBest, 1, 1, 1, 1, 1, 0)
			else
				DT.tooltip:AddDoubleLine(bgs[i], rating, 1, 1, 1, 1, 1, 0)
			end
		end
		DT.tooltip:AddLine(" ")
	end
	
	DT.tooltip:AddDoubleLine(L["Click"], L["Toggle Currency Frame"], 1, 1, 1, 1, 1, 0)
	
	if E.db.currencydt.tooltip.ratings == true then
		DT.tooltip:AddDoubleLine(L["Hold Shift"], L["Weekly Ratings"], 1, 1, 1, 1, 1, 0)
		DT.tooltip:AddDoubleLine(L["Hold Ctrl"], L["Season Ratings"], 1, 1, 1, 1, 1, 0)
	end
	DT.tooltip:Show()
end

local function OnClick(self, button)
	ToggleCharacter("TokenFrame")
end

local function ValueColorUpdate(hex, r, g, b)
	--displayString = join("", "%s")
	hexColor = hex
	if lastPanel ~= nil then OnEvent(lastPanel) end
end
E["valueColorUpdateFuncs"][ValueColorUpdate] = true

P["currencydt"] = {
	["display"] = 2,
	["icon"] = true,
	["showText"] = true,
	["showValue"] = true,
	["tooltip"] = {
		["headers"] = true,
		["zeroCurrency"] = "gray",
		["ratings"] = false,
		["colorWatched"] = true,
		["watchedColor"] = {r = 0, g = 1.0, b = 0},
	},
	["headers"] = {},
}

local function InjectOptions()
	if not E.Options.args.lockslap then
		E.Options.args.lockslap = {
			type = "group",
			order = -2,
			name = L["Plugins by |cff9382c9Lockslap|r"],
			args = {
				thanks = {
					type = "description",
					order = 1,
					name = L["Thanks for using and supporting my work!  -- |cff9382c9Lockslap|r\n\n|cffff0000If you find any bugs, or have any suggestions for any of my addons, please open a ticket at that particular addon's page on CurseForge."],
				},
			},
		}
	elseif not E.Options.args.lockslap.args.thanks then
		E.Options.args.lockslap.args.thanks = {
			type = "description",
			order = 1,
			name = L["Thanks for using and supporting my work!  -- |cff9382c9Lockslap|r\n\n|cffff0000If you find any bugs, or have any suggestions for any of my addons, please open a ticket at that particular addon's page on CurseForge."],
		}
	end

	E.Options.args.lockslap.args.currencydt = {
		type = "group",
		name = L["Currency Datatext"],
		get = function(info) return E.db.currencydt[info[#info]] end,
		set = function(info, value) E.db.currencydt[info[#info]] = value; DT:LoadDataTexts() end,
		args = {
			datatext = {
				type = "group",
				order = 1,
				name = L["Datatext Options"],
				guiInline = true,
				get = function(info) return E.db.currencydt[info[#info]] end,
				set = function(info, value) E.db.currencydt[info[#info]] = value; DT:LoadDataTexts() end,
				args = {
					display = {
						type = "select",
						order = 1,
						name = L["Display Currency"],
						desc = L["Currency to display in the datatext."],
						values = function()
							local temp = {}
							for i = 1, GetCurrencyListSize() do
								local name, isHeader, _, isUnused, isWatched, count, extraCurrencyType, icon, itemID = GetCurrencyListInfo(i)
								if not isHeader then
									temp[i] = name
								end
							end
							return temp
						end,
					},
					dummy = {
						type = "description",
						name = "",
						order = 2,
						width = "double",
					},
					icon = {
						type = "toggle",
						order = 3,
						name = L["Show Icon"],
						desc = L["Show the display currency's icon on the datatext."],
					},
					showText = {
						type = "toggle", 
						order = 4,
						name = L["Show Text"],
						desc = L["Show the display currency's text on the datatext."],
					},
					showValue = {
						type = "toggle",
						order = 5,
						name = L["Show Count"],
						desc = L["Show the display currency's count on the datatext."],
					},
				},
			},
			headers = {
				type = "multiselect",
				order = 2,
				name = L["Toggle Currency Headers"],
				desc = L["Select which currency headers to show."],
				get = function(info, key) return E.db.currencydt.headers[key] end,
				set = function(info, key, value) E.db.currencydt.headers[key] = value; DT:LoadDataTexts() end,
				values = function()
					local headers = {}
					for i = 1, GetCurrencyListSize() do
						local name, isHeader = GetCurrencyListInfo(i)
						if isHeader then
							headers[name] = name
						end
					end
					return headers
				end,
			},
			tooltip = {
				type = "group",
				order = 3,
				name = L["Tooltip Options"],
				guiInline = true,
				get = function(info) return E.db.currencydt.tooltip[info[#info]] end,
				set = function(info, value) E.db.currencydt.tooltip[info[#info]] = value; DT:LoadDataTexts() end,
				args = {
					headers = {
						type = "toggle",
						order = 1,
						name = L["Show Headers"],
						desc = L["Show the currency list headers."],
					},
					zeroCurrency = {
						type = "select",
						order = 2,
						name = L["Zero Currency Handling"],
						desc = L["How to handle currencies of which you have none of, but have discovered."],
						values = {
							["none"] = L["Nothing"],
							["hide"] = L["Hide"],
							["gray"] = L["Gray Out"],
						},
					},
					ratings = {
						type = "toggle",
						order = 3,
						name = L["Show Arena/RBG Ratings"],
						desc = L["Show your arena and RBG ratings in the tooltip."],
					},
					colorWatched = {
						type = "toggle",
						order = 4,
						name = L["Color Watched"],
						desc = L["Color the currencies being watched.  (Shown on Backpack)"],
					},
					watchedColor = {
						type = "color",
						order = 5,
						width = "double",
						disabled = function() return not E.db.currencydt.tooltip.colorWatched end,
						name = L["Watched Color"],
						desc = L["Color to use for the watched currencies."],
						get = function()
							return E.db.currencydt.tooltip.watchedColor.r,
								E.db.currencydt.tooltip.watchedColor.g,
								E.db.currencydt.tooltip.watchedColor.b
						end,
						set = function(_, r, g, b)
							E.db.currencydt.tooltip.watchedColor.r = r
							E.db.currencydt.tooltip.watchedColor.g = g
							E.db.currencydt.tooltip.watchedColor.b = b
							DT:LoadDataTexts()
						end,
					},
				},
			},
		},
	}
end

EP:RegisterPlugin(..., InjectOptions)
DT:RegisterDatatext("Currency", {"PLAYER_ENTERING_WORLD", "CHAT_MSG_CURRENCY", "CHAT_MSG_LOOT"}, OnEvent, nil, OnClick, OnEnter)